<?php
/**
 * @package Blue Hat CDN
 * @version 2.1
 * @copyright (C) Copyright 2006-2014 Blue Hat Network, BlueHatNetwork.com. All rights reserved.
 * @license GNU/GPL http://www.gnu.org/licenses/gpl-3.0.txt

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

class BlueHatNetworkNetwork
{
	private static $_ch = null;
	
	private static function _do($url, $params=null, $keepConnectionOpen=false, $saveToFileHandler=null)
	{
		//open connection
		if(empty(self::$_ch)) 
		{
			self::$_ch = curl_init();
			
			if($keepConnectionOpen)
			{
				$httpHeaders = array('Connection: Keep-Alive', 'Keep-Alive: 300', 'Expect:');
			}
			else
			{
				$httpHeaders = array('Expect:');
			}
			
			curl_setopt(self::$_ch, CURLOPT_HTTPHEADER, $httpHeaders);
			curl_setopt(self::$_ch, CURLOPT_RETURNTRANSFER, true);	
			curl_setopt(self::$_ch, CURLOPT_FOLLOWLOCATION, true);
			curl_setopt(self::$_ch, CURLOPT_SSL_VERIFYPEER, true);
			curl_setopt(self::$_ch, CURLOPT_ENCODING, 'gzip');
			curl_setopt(self::$_ch, CURLOPT_CONNECTTIMEOUT, 20);
			
			$cURLCaCertFilePath = BHN_PLUGIN_ADMIN_ROOT.DIRECTORY_SEPARATOR.'share'.DIRECTORY_SEPARATOR.'cacert.pem';
			
			if(is_file($cURLCaCertFilePath)) curl_setopt(self::$_ch, CURLOPT_CAINFO, $cURLCaCertFilePath);
		}
		
		curl_setopt(self::$_ch, CURLOPT_HTTPGET, true);
		curl_setopt(self::$_ch, CURLOPT_URL, $url);
		
		if(!empty($saveToFileHandler)) curl_setopt(self::$_ch, CURLOPT_FILE, $saveToFileHandler);
		
		if(!empty($params))
		{
			// This is a post
			curl_setopt(self::$_ch, CURLOPT_HTTPGET, false);
			curl_setopt(self::$_ch, CURLOPT_POST, true);
			curl_setopt(self::$_ch, CURLOPT_POSTFIELDS, $params);
		}

		//execute post
		$result = curl_exec(self::$_ch);

		//close connection
		if(!$keepConnectionOpen) self::closeConnection();
		
		return $result;
	}
	
	public static function get($url, $keepConnectionOpen=false)
	{
		return self::_do($url, null, $keepConnectionOpen);
	}
	
	public static function post($url, $params, $keepConnectionOpen=false)
	{
		return self::_do($url, $params, $keepConnectionOpen);
	}
	
	public static function closeConnection()
	{
		if(!empty(self::$_ch)) 
		{
			curl_close(self::$_ch);
			
			self::$_ch = null;
		}
	}
	
	public static function didErrorOccurWithLastRequest()
	{
		if(!empty(self::$_ch)) if(curl_errno(self::$_ch) || (int)curl_getinfo(self::$_ch, CURLINFO_HTTP_CODE) >= 400) return true;
		
		return false;
	}
	
	public static function downloadRemoteFile($remotePath, $localPath)
	{
		$didErrorOccur = true;
		
		$saveToFileHandler = fopen($localPath, 'w+');
		
		if($saveToFileHandler)
		{
			self::_do($remotePath, null, true, $saveToFileHandler);
		
			$didErrorOccur = self::didErrorOccurWithLastRequest();

			self::closeConnection();
			
			fclose($saveToFileHandler);
		}
		
		if(!$didErrorOccur)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
}