<?php
/**
 * @package Blue Hat CDN
 * @version 2.1
 * @copyright (C) Copyright 2006-2014 Blue Hat Network, BlueHatNetwork.com. All rights reserved.
 * @license GNU/GPL http://www.gnu.org/licenses/gpl-3.0.txt

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

if(!class_exists('BlueHatCDNModelBlueHatCDN')) require BHN_PLUGIN_ADMIN_ROOT.DIRECTORY_SEPARATOR.'models'.DIRECTORY_SEPARATOR.'bluehatcdn.php';

class BlueHatCDNModelBlueHatCDNBHN extends BlueHatCDNModelBlueHatCDN 
{
	private $_apiUri = 'https://api.bluehatnetwork.com/api/index.php?option=com_bluehatnetwork&view=api';
	private $_filesSynced = array();
	
	final public function __construct($html='')
	{
		parent::__construct($html);
	}
	
	final public function validateCDNAPICredentials($cdnProvider, $cdnUsername, $cdnAPIKey)
	{
		if(!class_exists('BlueHatNetworkNetwork')) require BHN_PLUGIN_ADMIN_ROOT.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'network.php';
		if(!class_exists('BlueHatNetworkSerializer')) require BHN_PLUGIN_ADMIN_ROOT.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'serializer.php';
		
		$response = BlueHatNetworkNetwork::post($this->_apiUri, array(
			'task' => 'bucket_info', 
			'api_username' => $cdnUsername, 
			'bucket_api_key' => $cdnAPIKey)
		);
		
		if(!empty($response))
		{
			$response = (array)BlueHatNetworkSerializer::decode($response);
			
			if(isset($response['bucket_path'])) if(!empty($response['bucket_path'])) return $response;
		}
		
		return false;
	}
	
	final public function syncFiles($allFiles)
	{
		$this->_filesSynced = array();
		
		if(!class_exists('BlueHatNetworkFactory')) require BHN_PLUGIN_ADMIN_ROOT.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'factory.php';
		if(!class_exists('BlueHatNetworkSetting')) require BHN_PLUGIN_ADMIN_ROOT.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		if(!class_exists('BlueHatNetworkLanguage')) require BHN_PLUGIN_ADMIN_ROOT.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'language.php';
		if(!class_exists('BlueHatNetworkNetwork')) require BHN_PLUGIN_ADMIN_ROOT.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'network.php';
		if(!class_exists('BlueHatNetworkSerializer')) require BHN_PLUGIN_ADMIN_ROOT.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'serializer.php';
		if(!class_exists('BlueHatNetworkDatabase')) require BHN_PLUGIN_ADMIN_ROOT.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'database.php';
		if(!class_exists('BlueHatNetworkDateTime')) require BHN_PLUGIN_ADMIN_ROOT.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'datetime.php';
		if(!class_exists('SmushIt')) require BHN_PLUGIN_ADMIN_ROOT.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'minify'.DIRECTORY_SEPARATOR.'SmushIt.class.php';
		
		if(!empty($allFiles))
		{
			$cdnProvider = BlueHatNetworkSetting::get('cdn_provider');
			$cdnUsername = BlueHatNetworkSetting::get('cdn_username');
			$cdnAPIKey = BlueHatNetworkSetting::get('cdn_api_key');
			
			$numOfFilesToSync = count($allFiles);
			
			$this->_touchLockFile(5);
			
			// First lets validate CDN credentials make sure they are valid
			if(!$this->validateCDNAPICredentials($cdnProvider, $cdnUsername, $cdnAPIKey))
			{
				$this->_setStatusMessage(BlueHatNetworkLanguage::_('BHN_COULD_NOT_VALIDATE_API_CREDENTIALS_ERROR'));
				
				$this->_slowDown();
				
				return false;
			}
			
			$this->_touchLockFile(20);
			
			$this->ensureCrossDomainXmlFileExistsOnCDN();
			
			$this->_touchLockFile();
			
			$currentUnixTimeStamp = BlueHatNetworkDateTime::getCurrentUnixStamp();
			
			foreach($allFiles as $key => $syncFile)
			{
				$this->_touchLockFile();
				
				if(is_file($syncFile))
				{
					$fullLocalPath = $syncFile;
				}
				else
				{
					$fullLocalPath = BlueHatNetworkFactory::getSiteFrontEndBasePath().$syncFile;
				}
				
				if(is_numeric($key))
				{
					$originalFileFullPath = $syncFile;
				}
				else
				{
					$originalFileFullPath = $key;
				}
				
				$originalFileStats = $this->getFileOriginalChecksum($originalFileFullPath);
				
				if($numOfFilesToSync > 1)
				{
					$doSyncForThisFile = false;
				}
				else
				{
					$doSyncForThisFile = true;
				}
				
				if(!empty($originalFileStats))
				{
					// File is indexed in DB, check checksum
					if(empty($originalFileStats['file_final_md5']) || $originalFileStats['file_original_md5'] != md5_file($fullLocalPath)) $doSyncForThisFile = true;
				}
				else
				{
					// File doesnt even exist in DB
					$doSyncForThisFile = true;
				}

				if($doSyncForThisFile) 
				{
					$remotePath = $originalFileFullPath;

					$optimizedFilePath = BlueHatNetworkFactory::getTmpPath().DIRECTORY_SEPARATOR.basename($originalFileFullPath);

					if($fullLocalPath != $optimizedFilePath)
					{
						if(is_file($optimizedFilePath)) unlink($optimizedFilePath);

						if(!copy($fullLocalPath, $optimizedFilePath)) continue;
					}

					$optimizationAttemptFailed = false;

					// First attempt optimize filesize
					if(preg_match('@\.js$@i', $syncFile))
					{
						// JavaScript file
						if(!file_put_contents($optimizedFilePath, file_get_contents($fullLocalPath))) $optimizationAttemptFailed = true;
					}
					elseif(preg_match('@\.css$@i', $syncFile))
					{
						// Css file
						if(!file_put_contents($optimizedFilePath, file_get_contents($fullLocalPath))) $optimizationAttemptFailed = true;
					}
					elseif(preg_match('@\.(?:png|gif|jpe?g|ico)$@i', $syncFile))
					{
						// Image file lossless optimization
						try {
							$smushit = new SmushIt(array($optimizedFilePath), SmushIt::THROW_EXCEPTION);

							foreach($smushit->get() as $optimizedImg) 
							{
								if(!empty($optimizedImg) && is_array($optimizedImg)) $optimizedImg = current($optimizedImg);
								
								if(empty($optimizedImg) || !is_object($optimizedImg) || !isset($optimizedImg->destination)) $optimizationAttemptFailed = true;
								if(!$optimizationAttemptFailed) if(!copy($optimizedImg->destination, $optimizedFilePath)) $optimizationAttemptFailed = true;
							}
						} catch(Exception $e) {
							$optimizationAttemptFailed = true;
						}
					}
					
					$this->_touchLockFile();
					
					clearstatcache(true, $fullLocalPath);
					clearstatcache(true, $optimizedFilePath);
					
					if($optimizedFilePath != $fullLocalPath && ($optimizationAttemptFailed || filesize($optimizedFilePath) < 1 || filesize($optimizedFilePath) > filesize($fullLocalPath))) copy($fullLocalPath, $optimizedFilePath);

					$postResult = BlueHatNetworkNetwork::post($this->_apiUri, array(
						'task' => 'put_file',
						'api_username' => $cdnUsername,
						'bucket_api_key' => $cdnAPIKey,
						'file_path' => $remotePath,
						'file_name' => '@'.$optimizedFilePath
					), true);
					
					$this->_touchLockFile();
					
					if(!empty($postResult))
					{
						$postResult = (array)BlueHatNetworkSerializer::decode($postResult);

						if(!empty($postResult) && (int)@$postResult['success'] == 1)
						{
							// Post was a success
							$sql = 'UPDATE '.BlueHatNetworkDatabase::getTablePrefix().'_file SET 
										file_final_md5 = '.BlueHatNetworkDatabase::quote($postResult['file_final_checksum']).',
										file_final_filesize = '.(int)$postResult['file_final_filesize'].', 
										file_mdate = '.$currentUnixTimeStamp.' 
									WHERE file_full_path = '.BlueHatNetworkDatabase::quote($originalFileFullPath).';';
							BlueHatNetworkDatabase::query($sql);

							$this->_syncFileCount++;

							if($this->_syncFileCount > 1)
							{
								if(!$this->_supressStatusMessageUpdates) $this->_setStatusMessage(BlueHatNetworkLanguage::sprintf('BHN_SYNCING_FILES', $this->_syncFileCount, $originalFileFullPath));
								
								$this->_setLastFileProcessed($originalFileFullPath);
							}
							else
							{
								if(!$this->_supressStatusMessageUpdates) $this->_setStatusMessage(BlueHatNetworkLanguage::sprintf('BHN_SYNCING_FILE', $this->_syncFileCount, $originalFileFullPath));
							}

							$this->_filesSynced[] = $originalFileFullPath;
						}
					}
					
					$this->_touchLockFile();
					
					if($fullLocalPath != $optimizedFilePath && is_file($optimizedFilePath)) @unlink($optimizedFilePath); // delete tmp optimized file when done with it
				}
				
				if($numOfFilesToSync > 1 && !$this->_doesLockFileExist()) BlueHatNetworkFactory::exitNow();
			}

			BlueHatNetworkNetwork::closeConnection();
		}
		
		return $this->_filesSynced;
	}
	
	final public function deleteCDNFile($filePathToDelete)
	{
		if(!class_exists('BlueHatNetworkSetting')) require BHN_PLUGIN_ADMIN_ROOT.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		if(!class_exists('BlueHatNetworkNetwork')) require BHN_PLUGIN_ADMIN_ROOT.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'network.php';
		if(!class_exists('BlueHatNetworkSerializer')) require BHN_PLUGIN_ADMIN_ROOT.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'serializer.php';
		
		$cdnProvider = BlueHatNetworkSetting::get('cdn_provider');
		$cdnUsername = BlueHatNetworkSetting::get('cdn_username');
		$cdnAPIKey = BlueHatNetworkSetting::get('cdn_api_key');
		
		if(empty($cdnProvider) || empty($cdnUsername) || empty($cdnAPIKey) || empty($filePathToDelete)) return false;
		
		$postResult = BlueHatNetworkNetwork::post($this->_apiUri, array(
			'task' => 'delete_file_from_bucket',
			'api_username' => $cdnUsername,
			'bucket_api_key' => $cdnAPIKey,
			'file_path' => $filePathToDelete
		), true);
		
		if(!empty($postResult))
		{
			$postResult = (array)BlueHatNetworkSerializer::decode($postResult);
			
			if((int)@$postResult['success'] == 1) return true;
		}
		
		return false;
	}
	
	public function ensureCrossDomainXmlFileExistsOnCDN()
	{
		if(!class_exists('BlueHatNetworkNetwork')) require BHN_PLUGIN_ADMIN_ROOT.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'network.php';
		if(!class_exists('BlueHatNetworkSerializer')) require BHN_PLUGIN_ADMIN_ROOT.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'serializer.php';
		if(!class_exists('BlueHatNetworkSetting')) require BHN_PLUGIN_ADMIN_ROOT.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'setting.php';
		if(!class_exists('BlueHatNetworkFactory')) require BHN_PLUGIN_ADMIN_ROOT.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'factory.php';
		
		$cdnProvider = BlueHatNetworkSetting::get('cdn_provider');
		$cdnUsername = BlueHatNetworkSetting::get('cdn_username');
		$cdnAPIKey = BlueHatNetworkSetting::get('cdn_api_key');
		
		if(empty($cdnProvider) || empty($cdnUsername) || empty($cdnAPIKey)) return false;
		
		$remoteFilePath = '/crossdomain.xml';
		
		$fileCheckResult = BlueHatNetworkNetwork::post($this->_apiUri, array(
			'task' => 'get_file_info',
			'api_username' => $cdnUsername,
			'bucket_api_key' => $cdnAPIKey,
			'file_path' => $remoteFilePath
		), true);
		
		if(!empty($fileCheckResult))
		{
			$fileCheckResult = (array)BlueHatNetworkSerializer::decode($fileCheckResult);
			
			if(!empty($fileCheckResult))
			{
				if((int)$fileCheckResult['success'] < 1)
				{
					$remoteFileContents = <<<EOF
<?xml version="1.0"?>
<cross-domain-policy>
	<allow-access-from domain="*"/>
</cross-domain-policy>
EOF;
					$localFilePath = BlueHatNetworkFactory::getTmpPath().DIRECTORY_SEPARATOR.basename($remoteFilePath);
					
					if(file_put_contents($localFilePath, $remoteFileContents))
					{
						$postResult = BlueHatNetworkNetwork::post($this->_apiUri, array(
							'task' => 'put_file',
							'api_username' => $cdnUsername,
							'bucket_api_key' => $cdnAPIKey,
							'file_path' => $remoteFilePath,
							'file_name' => '@'.$localFilePath
						), true);
						
						$this->_disableErrorReporting();
						
						@unlink($localFilePath);
						
						$this->_restoreErrorReporting();
						
						return true;
					}
				}
			}
		}
		
		return false;
	}
}